@@ -27,6 +27,9 @@
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #include <string.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/hash.h>
@@ -1047,7 +1050,7 @@ xmlAddXMLCatalog(xmlCatalogEntryPtr catal, const xmlChar *type,
 /**
  * xmlDelXMLCatalog:
  * @catal:  top of an XML catalog
- * @value:  the value to remove from teh catalog
+ * @value:  the value to remove from the catalog
  *
  * Remove entries in the XML catalog where the value or the URI
  * is equal to @value
@@ -1716,7 +1719,7 @@ xmlGetSGMLCatalogEntryType(const xmlChar *name) {
 }
 
 static int
-xmlParseSGMLCatalog(const xmlChar *value, const char *file) {
+xmlParseSGMLCatalog(const xmlChar *value, const char *file, int super) {
     const xmlChar *cur = value;
     xmlChar *base = NULL;
     int res;
@@ -1871,12 +1874,23 @@ xmlParseSGMLCatalog(const xmlChar *value, const char *file) {
 		}
 
 	    } else if (type == SGML_CATA_CATALOG) {
-		xmlChar *filename;
+		if (super) {
+		    xmlCatalogEntryPtr entry;
 
-		filename = xmlBuildURI(sysid, base);
-		if (filename != NULL) {
-		    xmlLoadCatalog((const char *)filename);
-		    xmlFree(filename);
+		    entry = xmlNewCatalogEntry(type, sysid, NULL,
+			                       XML_CATA_PREFER_NONE);
+		    res = xmlHashAddEntry(xmlDefaultCatalog, sysid, entry);
+		    if (res < 0) {
+			xmlFreeCatalogEntry(entry);
+		    }
+		} else {
+		    xmlChar *filename;
+
+		    filename = xmlBuildURI(sysid, base);
+		    if (filename != NULL) {
+			xmlLoadCatalog((const char *)filename);
+			xmlFree(filename);
+		    }
 		}
 	    }
 	    /*
@@ -2001,6 +2015,91 @@ xmlInitializeCatalog(void) {
     xmlCatalogInitialized = 1;
 }
 
+/**
+ * xmlLoadSGMLSuperCatalog:
+ * @filename:  a file path
+ *
+ * Load an SGML super catalog. It won't expand CATALOG or DELEGATE
+ * references. This is only needed for manipulating SGML Super Catalogs
+ * like adding and removing CATALOG or DELEGATE entries.
+ *
+ * Returns 0 in case of success -1 in case of error
+ */
+int
+xmlLoadSGMLSuperCatalog(const char *filename)
+{
+#ifdef HAVE_STAT
+    int fd;
+#else
+    FILE *fd;
+#endif
+    int len, ret;
+    long size;
+
+#ifdef HAVE_STAT
+    struct stat info;
+#endif
+    xmlChar *content;
+
+    if (filename == NULL)
+        return (-1);
+
+    if (xmlDefaultCatalog == NULL)
+        xmlDefaultCatalog = xmlHashCreate(20);
+    if (xmlDefaultCatalog == NULL)
+        return (-1);
+
+#ifdef HAVE_STAT
+    if (stat(filename, &info) < 0)
+        return (-1);
+#endif
+
+#ifdef HAVE_STAT
+    if ((fd = open(filename, O_RDONLY)) < 0) {
+#else
+    if ((fd = fopen(filename, "rb")) == NULL) {
+#endif
+        catalNr--;
+        return (-1);
+    }
+#ifdef HAVE_STAT
+    size = info.st_size;
+#else
+    if (fseek(fd, 0, SEEK_END) || (size = ftell(fd)) == EOF || fseek(fd, 0, SEEK_SET)) {        /* File operations denied? ok, just close and return failure */
+        fclose(fd);
+        return (-1);
+    }
+#endif
+    content = xmlMalloc(size + 10);
+    if (content == NULL) {
+        xmlGenericError(xmlGenericErrorContext,
+                        "realloc of %d byte failed\n", size + 10);
+        catalNr--;
+        return (-1);
+    }
+#ifdef HAVE_STAT
+    len = read(fd, content, size);
+#else
+    len = fread(content, 1, size, fd);
+#endif
+    if (len < 0) {
+        xmlFree(content);
+        catalNr--;
+        return (-1);
+    }
+    content[len] = 0;
+#ifdef HAVE_STAT
+    close(fd);
+#else
+    fclose(fd);
+#endif
+
+    ret = xmlParseSGMLCatalog(content, filename, 1);
+
+    xmlFree(content);
+    return (ret);
+}
+
 /**
  * xmlLoadCatalog:
  * @filename:  a file path
@@ -2013,95 +2112,126 @@ xmlInitializeCatalog(void) {
  * Returns 0 in case of success -1 in case of error
  */
 int
-xmlLoadCatalog(const char *filename) {
-    int fd, len, ret, i;
+xmlLoadCatalog(const char *filename)
+{
+#ifdef HAVE_STAT
+    int fd;
+#else
+    FILE *fd;
+#endif
+    int len, ret, i;
+    long size;
+
+#ifdef HAVE_STAT
     struct stat info;
+#endif
     xmlChar *content;
 
     if (filename == NULL)
-	return(-1);
+        return (-1);
 
     if (xmlDefaultCatalog == NULL)
-	xmlDefaultCatalog = xmlHashCreate(20);
+        xmlDefaultCatalog = xmlHashCreate(20);
     if (xmlDefaultCatalog == NULL)
-	return(-1);
+        return (-1);
 
     /*
      * Need to be done after ...
      */
     if (!xmlCatalogInitialized)
-	xmlInitializeCatalog();
+        xmlInitializeCatalog();
 
 #ifdef HAVE_STAT
-    if (stat(filename, &info) < 0) 
-	return(-1);
+    if (stat(filename, &info) < 0)
+        return (-1);
 #endif
 
     /*
      * Prevent loops
      */
-    for (i = 0;i < catalNr;i++) {
-	if (xmlStrEqual((const xmlChar *)catalTab[i],
-		        (const xmlChar *)filename)) {
-	    xmlGenericError(xmlGenericErrorContext,
-		"xmlLoadCatalog: %s seems to induce a loop\n",
-		            filename);
-	    return(-1);
-	}
+    for (i = 0; i < catalNr; i++) {
+        if (xmlStrEqual((const xmlChar *) catalTab[i],
+                        (const xmlChar *) filename)) {
+            xmlGenericError(xmlGenericErrorContext,
+                            "xmlLoadCatalog: %s seems to induce a loop\n",
+                            filename);
+            return (-1);
+        }
     }
     if (catalNr >= catalMax) {
-	xmlGenericError(xmlGenericErrorContext,
-	    "xmlLoadCatalog: %s catalog list too deep\n",
-			filename);
-	    return(-1);
+        xmlGenericError(xmlGenericErrorContext,
+                        "xmlLoadCatalog: %s catalog list too deep\n",
+                        filename);
+        return (-1);
     }
     catalTab[catalNr++] = filename;
 
+#ifdef HAVE_STAT
     if ((fd = open(filename, O_RDONLY)) < 0) {
-	catalNr--;
-	return(-1);
+#else
+    if ((fd = fopen(filename, "rb")) == NULL) {
+#endif
+        catalNr--;
+        return (-1);
     }
-
-    content = xmlMalloc(info.st_size + 10);
+#ifdef HAVE_STAT
+    size = info.st_size;
+#else
+    if (fseek(fd, 0, SEEK_END) || (size = ftell(fd)) == EOF || fseek(fd, 0, SEEK_SET)) {        /* File operations denied? ok, just close and return failure */
+        fclose(fd);
+        return (-1);
+    }
+#endif
+    content = xmlMalloc(size + 10);
     if (content == NULL) {
-	xmlGenericError(xmlGenericErrorContext,
-		"realloc of %d byte failed\n", info.st_size + 10);
-	catalNr--;
-	return(-1);
+        xmlGenericError(xmlGenericErrorContext,
+                        "realloc of %d byte failed\n", size + 10);
+        catalNr--;
+        return (-1);
     }
-    len = read(fd, content, info.st_size);
+#ifdef HAVE_STAT
+    len = read(fd, content, size);
+#else
+    len = fread(content, 1, size, fd);
+#endif
     if (len < 0) {
-	xmlFree(content);
-	catalNr--;
-	return(-1);
+        xmlFree(content);
+        catalNr--;
+        return (-1);
     }
     content[len] = 0;
+#ifdef HAVE_STAT
     close(fd);
+#else
+    fclose(fd);
+#endif
 
     if ((content[0] == ' ') || (content[0] == '-') ||
-	((content[0] >= 'A') && (content[0] <= 'Z')) ||
-	((content[0] >= 'a') && (content[0] <= 'z')))
-	ret = xmlParseSGMLCatalog(content, filename);
+        ((content[0] >= 'A') && (content[0] <= 'Z')) ||
+        ((content[0] >= 'a') && (content[0] <= 'z')))
+        ret = xmlParseSGMLCatalog(content, filename, 0);
     else {
-	xmlCatalogEntryPtr catal, tmp;
-	/* TODO: allow to switch the default preference */
-	catal = xmlParseXMLCatalog(content, XML_CATA_PREFER_PUBLIC, filename);
-	if (catal != NULL) {
-	    if (xmlDefaultXMLCatalogList == NULL)
-		xmlDefaultXMLCatalogList = catal;
-	    else {
-		tmp = xmlDefaultXMLCatalogList;
-		while (tmp->next != NULL)
-		    tmp = tmp->next;
-		tmp->next = catal;
-	    }
-	    ret = 0;
-	} else
-	    ret = -1;
+        xmlCatalogEntryPtr catal, tmp;
+
+        /* TODO: allow to switch the default preference */
+        catal =
+            xmlParseXMLCatalog(content, XML_CATA_PREFER_PUBLIC, filename);
+        if (catal != NULL) {
+            if (xmlDefaultXMLCatalogList == NULL)
+                xmlDefaultXMLCatalogList = catal;
+            else {
+                tmp = xmlDefaultXMLCatalogList;
+                while (tmp->next != NULL)
+                    tmp = tmp->next;
+                tmp->next = catal;
+            }
+            ret = 0;
+        } else
+            ret = -1;
     }
     xmlFree(content);
     catalNr--;
-    return(ret);
+    return (ret);
 }
 
 /**
@@ -2431,6 +2561,15 @@ int
 xmlCatalogAdd(const xmlChar *type, const xmlChar *orig, const xmlChar *replace) {
     int res = -1;
 
+    if ((xmlDefaultCatalog != NULL) &&
+	(xmlStrEqual(type, BAD_CAST "sgmlcatalog"))) {
+	xmlCatalogEntryPtr entry;
+
+	entry = xmlNewCatalogEntry(SGML_CATA_CATALOG, replace, NULL,
+				   XML_CATA_PREFER_NONE);
+	res = xmlHashAddEntry(xmlDefaultCatalog, replace, entry);
+	return(0);
+    } 
     if ((xmlDefaultXMLCatalogList == NULL) &&
 	(xmlStrEqual(type, BAD_CAST "catalog"))) {
 	xmlDefaultXMLCatalogList = xmlNewCatalogEntry(XML_CATA_CATALOG, NULL,
@@ -2444,12 +2583,12 @@ xmlCatalogAdd(const xmlChar *type, const xmlChar *orig, const xmlChar *replace)
     if (xmlDefaultXMLCatalogList != NULL) {
 	res = xmlAddXMLCatalog(xmlDefaultXMLCatalogList, type, orig, replace);
     } else if (xmlDefaultCatalog != NULL) {
-	xmlCatalogEntryType typ;
+	xmlCatalogEntryType cattype;
 
-	typ = xmlGetSGMLCatalogEntryType(type);
-	if (type != XML_CATA_NONE) {
+	cattype = xmlGetSGMLCatalogEntryType(type);
+	if (cattype != XML_CATA_NONE) {
 	    xmlCatalogEntryPtr entry;
-	    entry = xmlNewCatalogEntry(typ, orig, replace,
+	    entry = xmlNewCatalogEntry(cattype, orig, replace,
 		                       XML_CATA_PREFER_NONE);
 	    res = xmlHashAddEntry(xmlDefaultCatalog, orig, entry);
 	}
@@ -2463,7 +2602,7 @@ xmlCatalogAdd(const xmlChar *type, const xmlChar *orig, const xmlChar *replace)
  *
  * Remove an entry from the catalog
  *
- * Returns 0 if successful, -1 otherwise
+ * Returns the number of entries removed if successful, -1 otherwise
  */
 int
 xmlCatalogRemove(const xmlChar *value) {
@@ -2475,7 +2614,10 @@ xmlCatalogRemove(const xmlChar *value) {
     if (xmlDefaultXMLCatalogList != NULL) {
 	res = xmlDelXMLCatalog(xmlDefaultXMLCatalogList, value);
     } else if (xmlDefaultCatalog != NULL) {
-	TODO
+	res = xmlHashRemoveEntry(xmlDefaultCatalog, value,
+		(xmlHashDeallocator) xmlFreeCatalogEntry);
+	if (res == 0)
+	    res = 1;
     } 
     return(res);
 }
