@@ -46,7 +46,7 @@ struct _xmlParserInput {
     xmlParserInputBufferPtr buf;      /* UTF-8 encoded buffer */
 
     const char *filename;             /* The file analyzed, if any */
-    const char *directory;            /* the directory/base of teh file */
+    const char *directory;            /* the directory/base of the file */
     const xmlChar *base;              /* Base of the array to parse */
     const xmlChar *cur;               /* Current char being parsed */
     const xmlChar *end;               /* end of the arry to parse */
@@ -456,6 +456,10 @@ xmlDocPtr	xmlSAXParseMemory	(xmlSAXHandlerPtr sax,
 xmlDocPtr	xmlSAXParseFile		(xmlSAXHandlerPtr sax,
 					 const char *filename,
 					 int recovery);
+xmlDocPtr	xmlSAXParseFileWithData	(xmlSAXHandlerPtr sax,
+					 const char *filename,
+					 int recovery,
+					 void *data);
 xmlDocPtr	xmlSAXParseEntity	(xmlSAXHandlerPtr sax,
 					 const char *filename);
 xmlDocPtr	xmlParseEntity		(const char *filename);
