@@ -922,7 +922,7 @@ xmlNormalizeURIPath(char *path) {
 /**
  * xmlURIUnescapeString:
  * @str:  the string to unescape
- * @len:   the lenght in bytes to unescape (or <= 0 to indicate full string)
+ * @len:   the length in bytes to unescape (or <= 0 to indicate full string)
  * @target:  optionnal destination buffer
  *
  * Unescaping routine, does not do validity checks !
@@ -1649,14 +1649,19 @@ xmlParseURIHierPart(xmlURIPtr uri, const char **str) {
 static int
 xmlParseAbsoluteURI(xmlURIPtr uri, const char **str) {
     int ret;
+    const char *cur;
 
     if (str == NULL)
 	return(-1);
     
+    cur = *str;
+
     ret = xmlParseURIScheme(uri, str);
     if (ret != 0) return(ret);
-    if (**str != ':')
+    if (**str != ':') {
+	*str = cur;
 	return(1);
+    }
     (*str)++;
     if (**str == '/')
 	return(xmlParseURIHierPart(uri, str));
