@@ -3,7 +3,7 @@
  *
  * See Copyright for the status of this software.
  *
- * Daniel.Veillard@w3.org
+ * daniel@veillard.com
  *
  * 14 Nov 2000 ht - for VMS, truncated name of long functions to under 32 char
  */
@@ -1595,6 +1595,9 @@ xmlParserInputBufferCreateFilename
 
     if (URI == NULL) return(NULL);
 
+#ifdef LIBXML_CATALOG_ENABLED
+#endif
+
     /*
      * Try to find one of the input accept method accepting taht scheme
      * Go in reverse to give precedence to user defined handlers.
@@ -2356,6 +2359,34 @@ xmlParserGetDirectory(const char *filename) {
  *								*
  ****************************************************************/
 
+#ifdef LIBXML_CATALOG_ENABLED
+static int xmlSysIDExists(const char *URL) {
+#ifdef HAVE_STAT
+    int ret;
+    struct stat info;
+    const char *path;
+
+    if (URL == NULL)
+	return(0);
+
+    if (!strncmp(URL, "file://localhost", 16))
+	path = &URL[16];
+    else if (!strncmp(URL, "file:///", 8)) {
+#ifdef _WIN32
+	path = &URL[8];
+#else
+	path = &URL[7];
+#endif
+    } else 
+	path = URL;
+    ret = stat(path, &info);
+    if (ret == 0)
+	return(1);
+#endif
+    return(0);
+}
+#endif
+
 /**
  * xmlDefaultExternalEntityLoader:
  * @URL:  the URL for the entity to load
@@ -2371,7 +2402,10 @@ xmlParserInputPtr
 xmlDefaultExternalEntityLoader(const char *URL, const char *ID,
                                xmlParserCtxtPtr ctxt) {
     xmlParserInputPtr ret = NULL;
-    const xmlChar *resource = NULL;
+    xmlChar *resource = NULL;
+#ifdef LIBXML_CATALOG_ENABLED
+    xmlCatalogAllow pref;
+#endif
 
 #ifdef DEBUG_EXTERNAL_ENTITIES
     xmlGenericError(xmlGenericErrorContext,
@@ -2379,16 +2413,61 @@ xmlDefaultExternalEntityLoader(const char *URL, const char *ID,
 #endif
 #ifdef LIBXML_CATALOG_ENABLED
     /*
-     * Try to load it from the resource pointed in the catalog
+     * If the resource doesn't exists as a file,
+     * try to load it from the resource pointed in the catalogs
      */
-    if (ID != NULL)
-	resource = xmlCatalogGetPublic((const xmlChar *)ID);
-    if ((resource == NULL) && (URL != NULL))
-	resource = xmlCatalogGetSystem((const xmlChar *)URL);
+    pref = xmlCatalogGetDefaults();
+
+    if ((pref != XML_CATA_ALLOW_NONE) && (!xmlSysIDExists(URL))) {
+	/*
+	 * Do a local lookup
+	 */
+	if ((ctxt->catalogs != NULL) &&
+	    ((pref == XML_CATA_ALLOW_ALL) ||
+	     (pref == XML_CATA_ALLOW_DOCUMENT))) {
+	    resource = xmlCatalogLocalResolve(ctxt->catalogs,
+					      (const xmlChar *)ID,
+					      (const xmlChar *)URL);
+        }
+	/*
+	 * Try a global lookup
+	 */
+	if ((resource == NULL) &&
+	    ((pref == XML_CATA_ALLOW_ALL) ||
+	     (pref == XML_CATA_ALLOW_GLOBAL))) {
+	    resource = xmlCatalogResolve((const xmlChar *)ID,
+					 (const xmlChar *)URL);
+	}
+	if ((resource == NULL) && (URL != NULL))
+	    resource = xmlStrdup((const xmlChar *) URL);
+
+	/*
+	 * TODO: do an URI lookup on the reference
+	 */
+	if ((resource != NULL) && (!xmlSysIDExists((const char *)resource))) {
+	    xmlChar *tmp = NULL;
+
+	    if ((ctxt->catalogs != NULL) &&
+		((pref == XML_CATA_ALLOW_ALL) ||
+		 (pref == XML_CATA_ALLOW_DOCUMENT))) {
+		tmp = xmlCatalogLocalResolveURI(ctxt->catalogs, resource);
+	    }
+	    if ((tmp == NULL) &&
+		((pref == XML_CATA_ALLOW_ALL) ||
+	         (pref == XML_CATA_ALLOW_GLOBAL))) {
+		tmp = xmlCatalogResolveURI(resource);
+	    }
+
+	    if (tmp != NULL) {
+		xmlFree(resource);
+		resource = tmp;
+	    }
+	}
+    }
 #endif
 
     if (resource == NULL)
-	resource = (const xmlChar *)URL;
+	resource = (xmlChar *) URL;
 
     if (resource == NULL) {
 	if ((ctxt->validate) && (ctxt->sax != NULL) && 
@@ -2410,6 +2489,8 @@ xmlDefaultExternalEntityLoader(const char *URL, const char *ID,
 	    ctxt->sax->warning(ctxt,
 		    "failed to load external entity \"%s\"\n", resource);
     }
+    if ((resource != NULL) && (resource != (xmlChar *) URL))
+	xmlFree(resource);
     return(ret);
 }
 
@@ -2442,7 +2523,7 @@ xmlGetExternalEntityLoader(void) {
 /**
  * xmlLoadExternalEntity:
  * @URL:  the URL for the entity to load
- * @ID:  the System ID for the entity to load
+ * @ID:  the Public ID for the entity to load
  * @ctxt:  the context in which the entity is called or NULL
  *
  * Load an external entity, note that the use of this function for
